#include "../Framework/Framework.h"
#include "../Framework/Util/ProceduralTerrain.h"
#include "../Framework/Math/PerlinNoise.h"
#include <stdio.h>

class MainApp : public OpenGLApp {
protected:
	TextureID ground;
	TextureID skyPosX, skyNegX, skyPosY, skyPosZ, skyNegZ;

	ProceduralTerrain *terrain;

public:
	bool init();
	bool load();
	bool unload();
	bool drawFrame();
	
	void resetCamera();
	void selectPixelFormat(PixelFormat &pixelFormat);
};

void MainApp::resetCamera(){
	position = Vertex(0,100,0);
	Wx = 0;
	Wy = 1.0f;
	Wz = 0;
}

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 0;

	pixelFormat.depthBits = 24;
}

float terrainFunc(const float x, const float z){
	//return 256 * sinf(x * 0.00734f) * cosf(z * 0.00234f);
	return 256 * noise2(0.0031212312f * x, 0.004613212f * z) + 32 * noise2(0.017412412f * x, 0.01752341f * z);
}

bool MainApp::init(){
	initPerlin();

	speed = 256;
	return true;
}

bool MainApp::load(){
	ground = renderer->addTexture("../Textures/Grass2.jpg");

	skyNegX = renderer->addTexture("../Textures/CubeMaps/FogSky/negx.jpg", TEX_CLAMP);
	skyPosX = renderer->addTexture("../Textures/CubeMaps/FogSky/posx.jpg", TEX_CLAMP);
	skyPosY = renderer->addTexture("../Textures/CubeMaps/FogSky/posy.jpg", TEX_CLAMP);
	skyNegZ = renderer->addTexture("../Textures/CubeMaps/FogSky/negz.jpg", TEX_CLAMP);
	skyPosZ = renderer->addTexture("../Textures/CubeMaps/FogSky/posz.jpg", TEX_CLAMP);

	textTexture  = renderer->addTexture("../Textures/font2.dds");

	projection.loadProjectionMatrix(90 * PI / 180, 0.75f, 1, 4000);

	terrain = new ProceduralTerrain(terrainFunc, 1024.0f, 128, 4, 140);

	return true;
}

bool MainApp::unload(){
	delete terrain;

	return true;
}

bool MainApp::drawFrame(){

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projection);

	modelView.loadRotateZXY(-Wx, -Wy, -Wz);

    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	glClearColor(58.0f / 255, 68.0f / 255, 184.0f / 255, 1.0f);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glDepthFunc(GL_LEQUAL);

	glDisable(GL_DEPTH_TEST);

	renderer->setTextures(skyNegX);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128,  128, -128);

		glTexCoord2f(1,0);
		glVertex3f(-128,  128,  128);

		glTexCoord2f(1,1);
		glVertex3f(-128, -128,  128);

		glTexCoord2f(0,1);
		glVertex3f(-128, -128, -128);
	glEnd();
	renderer->setTextures(skyPosX);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f( 128,  128,  128);

		glTexCoord2f(1,0);
		glVertex3f( 128,  128, -128);

		glTexCoord2f(1,1);
		glVertex3f( 128, -128, -128);

		glTexCoord2f(0,1);
		glVertex3f( 128, -128,  128);
	glEnd();
	/*renderer->setTextures(skyNegY);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128, -128,  128);

		glTexCoord2f(1,0);
		glVertex3f( 128, -128,  128);

		glTexCoord2f(1,1);
		glVertex3f( 128, -128, -128);

		glTexCoord2f(0,1);
		glVertex3f(-128, -128, -128);
	glEnd();*/
	renderer->setTextures(skyPosY);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128,  128, -128);

		glTexCoord2f(1,0);
		glVertex3f( 128,  128, -128);

		glTexCoord2f(1,1);
		glVertex3f( 128,  128,  128);

		glTexCoord2f(0,1);
		glVertex3f(-128,  128,  128);
	glEnd();
	renderer->setTextures(skyPosZ);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128,  128,  128);

		glTexCoord2f(1,0);
		glVertex3f( 128,  128,  128);

		glTexCoord2f(1,1);
		glVertex3f( 128, -128,  128);

		glTexCoord2f(0,1);
		glVertex3f(-128, -128,  128);
	glEnd();
	renderer->setTextures(skyNegZ);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f( 128,  128, -128);

		glTexCoord2f(1,0);
		glVertex3f(-128,  128, -128);

		glTexCoord2f(1,1);
		glVertex3f(-128, -128, -128);

		glTexCoord2f(0,1);
		glVertex3f( 128, -128, -128);
	glEnd();

	modelView.translate(-position);
    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	glEnable(GL_DEPTH_TEST);

    float fogColor[4] = { 58.0f / 255, 68.0f / 255, 184.0f / 255, 1.0f };

    glFogfv(GL_FOG_COLOR, fogColor);    
    glFogi(GL_FOG_MODE, GL_EXP2);
    glFogf(GL_FOG_DENSITY, 0.001f);
    //glFogf(GL_FOG_START, 1);
    //glFogf(GL_FOG_END, 4000);
    glEnable(GL_FOG);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	float lightDir[] = { -1.0f, 0.5f, 0.2f, 0.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, lightDir);

	renderer->setTextures(ground);
	//renderer->setTexGen(TEXGEN_OBJECT_LINEAR);
	renderer->apply();

	/*renderer->changeTextureUnit(0);
	glMatrixMode(GL_TEXTURE);
	float mat[16] = {
		0.005f, 0,      0, 0,
		0,      0,      0, 0,
		0,      0.005f, 0, 0,
		0,      0,      0, 1
	};
	glLoadMatrixf(mat);
	*/

	int nVertices = terrain->draw(renderer, position, 4000, 0.0013f);

	//glLoadIdentity();

	char str[64];
	sprintf(str, "%.1f Mt/s", (nVertices / time) / 1000000);
	setDebugString(str);

	glDisable(GL_LIGHTING);
    glDisable(GL_FOG);

	return true;
}

DECLARE_MAIN_APP(MainApp)
